#pragma once
#include <string>
#include <thread>
#include <iostream>
#include "BaseSettings.h"
#include "RawData.h"
#include "BaseDataExtractor.h"
#include "Callback.h"



using namespace std;

#ifdef STARFLEETTOOLBOX_EXPORTS
#define BASECOMM __declspec(dllexport)
#else
#define BASECOMM __declspec(dllimport)
#endif

#define MONITORING_TIMESLEEP 500		// Sets the loop period of the monitoring system

#define MONITORING_MODE unsigned int
#define MONITORING_OFF 0				// Indicates that monitoring is OFF
#define MONITORING_ON 1					// Indicates that monitoring is ON
#define AUTO_RECONNECT 2				// Indicates that monitoring is ON and auto reconnect mode is on

#define CONNECTION_LOST  3				// The connection has been lost
#define DISCONNECTING 4					// The system is disconnecting and clearing any ressources linked to physical connection
#define RECONNECTING 5					// The system is reconnecting and allocating ressources linged to physical connection
#define RECONNECTED 6					// The host has been reconnected
#define DISCONNECTED 7					// The host has been disconnected
#define CONNECTION_BACK 8				// The host connection is back
#define NO_CONNECTION  9				// There is no connection to the host
#define CONNECTED 10					// There is a connection to the host

/// <summary>
/// Basic abstract communication class. Must be implemented by subclasses in charge of communication
/// </summary>
class BASECOMM BaseCommunication abstract
{
public:
	/// <summary>
	/// Base constructor
	/// </summary>
	BaseCommunication();

	/// <summary>
	/// Base constructor
	/// </summary>
	BaseCommunication(MONITORING_MODE monitor);

	/// <summary>
	/// Destructor
	/// </summary>
	~BaseCommunication();

	/// <summary>
	/// Sets the communication settings
	/// </summary>
	/// <param name="settings">The communication settings to be set</param>
	void SetSettings(BaseSettings* settings);

	/// <summary>
	/// Gets the communication settings
	/// </summary>
	/// <returns>An encaspulation of communication settings</returns>
	BaseSettings* GetSettings();


	/// <summary>
	/// Sets the data extractor
	/// </summary>
	/// <param name="pDataExtractor">The data extractor to be set</param>
	void SetDataExtractor(BaseDataExtractor* pDataExtractor);

	/// <summary>
	/// Gets the data extractir
	/// </summary>
	/// <returns>A pointer to the data extractor</returns>
	BaseDataExtractor* GetDataExtractor();

	/// <summary>
	/// Pure virtual method that will help the caller to proceed to the connection
	/// to media
	/// </summary>
	virtual void Connect() = 0;


	/// <summary>
	/// Pure virtual method that will help the caller to proceed to media disconection
	/// </summary>
	virtual void Disconnect() = 0;

	/// <summary>
	/// Returns the actual connection state. /!\: monitoring must be turned to ON
	/// to get the realtime status. The default will be DISCONNECTED
	/// </summary>
	/// <returns>CONNECTION_LOST, DISCONNECTING, RECONNECTING, RECONNECTED, DISCONNECTED (default), CONNECTION_BACK, NO_CONNECTION, CONNECTED</returns>
	int GetConnectionState();

	/// <summary>
	/// Indicates rather or not connect or disconnect method has been called
	/// </summary>
	/// <returns>true if 'connect' has been called</returns>
	bool IsConnected();


	/// <summary>
	/// Pure virtual method that will help the caller to send command
	/// on the media
	/// </summary>
	/// <param name="commandToSend">A string representing the command to be sent</param>
	/// <returns></returns>
	virtual string SendCommand(string commandToSend) = 0;

	/// <summary>
	/// Pure virtual method that will help the caller to wait for RAW data comming
	/// from the media
	/// </summary>
	/// <param name="waitTimeout">The amount of miliseconds the process has to wait befor being released</param>
	/// <param name="semStatus">The status of the semaphore (updated by reference)</param>
	/// <returns>A pointer to RAW Data</returns>
	virtual RawData WaitForRawData(int waitTimeout, unsigned long* semStatus) = 0;	

	/// <summary>
	/// Allocate the callback structure for the caller program
	/// Allocating the callback structure is mandatoring prior using the monitoring
	/// </summary>
	/// <param name="caller">A pointer to the caller structure</param>
	void AllocCallbackCaller(CONNECTION_CALLBACK* caller);

	/// <summary>
	/// Sets the caller program static method that will be called by the HEARTBEAT internal process
	/// </summary>
	/// <param name="pCallback">A pointer to a static method</param>
	void SetHeartBeatCallback(void (*pCallback)(CONNECTION_CALLBACK*));

	/// <summary>
	/// Starts the monitoring process
	/// </summary>
	void StartMonitoring();
	
	/// <summary>
	/// Stops the monitoring process
	/// </summary>
	void StopMonitoring();


protected:
	bool _connectHasBeenCalled = false;
	volatile int _connectionState = DISCONNECTED;				/// Indicates the default value for connection state flag

	/// <summary>
	/// Sets the connection state
	/// <param name="connectionState">The connection state new value</param>
	/// </summary>
	void SetConnectionState(int connectionState);

	/// <summary>
	/// Returns the communication status
	/// </summary>
	/// <returns>Success = 1, 0 for failure</returns>
	virtual int GetCommunicationStatus() = 0;



	/// <summary>
	/// Returns the monitoring mode
	/// </summary>
	/// <returns>MONITORING_OFF, MONITORING_ON or AUTO_RECONNECT</returns>
	MONITORING_MODE GetMonitoringMode();
	

	/// <summary>
	/// Proceed to TCP and UDP connection semaphores and mutex creation and Threads launching
	/// force monitoring or not
	/// </summary>
	/// <param name="forceMoninoring">true: forces the monitoring check</param>
	/// <summary>
	virtual void Connect(bool forceMoninoring) = 0;

	void *_mutexMonitoring;								/// The mutex for the monitoring flag
	void* _mutexConnectState;							/// The mutex for the connection state value
	volatile int _connStatus = 0;						/// The real connection status given bythe monitoring process
	CONNECTION_CALLBACK* _callerCallback = NULL;		/// The caller callback structure that has to be created by caller program
private:


	MONITORING_MODE _monitoringMode = MONITORING_OFF;	/// Default monitoring mode
	thread _monitoringWorker;							/// The monitoring thread handle

	/// <summary>
	/// The monitoring thread business code
	/// </summary>
	void MonitoringBusiness(CONNECTION_CALLBACK *connCallback);

	bool _monitoringExit = false;						/// The flag to exit the monitoring thread
	void (*callback)(CONNECTION_CALLBACK*);				/// The internal pointer on the callback	

	BaseSettings* _settings;
	BaseDataExtractor* _dataExtractor;
};

